import { PLUGINS_CONTENT_ELEMENT_ATTR_NAME, PLUGINS_CONTENT_ELEMENT_ID } from '@/constants/plugins';
import { registerPlugin } from '@/redux/plugins/plugins.thunks';
import { store } from '@/redux/store';
import md5 from 'crypto-js/md5';
import { bioEntitiesMethods } from './bioEntities';
import { getModels } from './map/models/getModels';
import { openMap } from './map/openMap';
import { getCenter } from './map/position/getCenter';
import { setCenter } from './map/position/setCenter';
import { triggerSearch } from './map/triggerSearch';
import { getZoom } from './map/zoom/getZoom';
import { setZoom } from './map/zoom/setZoom';
import { getCurrentOverviewImage } from './overviewImage/getCurrentOverviewImage';
import { getOverviewImages } from './overviewImage/getOverviewImages';
import { hideOverviewImageModal } from './overviewImage/hideOverviewImageModal';
import { selectOverviewImage } from './overviewImage/selectOverviewImage';
import { showOverviewImageModal } from './overviewImage/showOverviewImageModal';
import { PluginsEventBus } from './pluginsEventBus';
import type { PluginsManagerType } from './pluginsManager.types';
import { configurationMapper } from './pluginsManager.utils';
import { getDataOverlays } from './map/overlays/getDataOverlays';
import { getVisibleDataOverlays } from './map/overlays/getVisibleDataOverlays';
import { showDataOverlay } from './map/overlays/showDataOverlay';
import { hideDataOverlay } from './map/overlays/hideDataOverlay';
import { removeDataOverlay } from './map/overlays/removeDataOverlay';
import { addDataOverlay } from './map/overlays/addDataOverlay';
import { getApiUrls } from './project/data/getApiUrls';
import { getDisease } from './project/data/getDisease';
import { getName } from './project/data/getName';
import { getOrganism } from './project/data/getOrganism';
import { getProjectId } from './project/data/getProjectId';
import { getVersion } from './project/data/getVersion';
import { getBounds } from './map/data/getBounds';
import { fitBounds } from './map/fitBounds';
import { getOpenMapId } from './map/getOpenMapId';
import { setLegend } from './legend/setLegend';
import { removeLegend } from './legend/removeLegend';
import { ERROR_PLUGIN_URL_MISMATCH } from './errorMessages';

export const PluginsManager: PluginsManagerType = {
  hashedPlugins: {},
  setHashedPlugin({ pluginUrl, pluginScript }) {
    const hash = md5(pluginScript).toString();

    PluginsManager.hashedPlugins[pluginUrl] = hash;

    return hash;
  },
  init() {
    window.minerva = {
      plugins: {
        registerPlugin: PluginsManager.registerPlugin,
      },
      data: {
        bioEntities: bioEntitiesMethods,
      },
      map: {
        data: {
          getBounds,
          getOpenMapId,
          getModels,
        },
        fitBounds,
        openMap,
        triggerSearch,
        getZoom,
        setZoom,
        getCenter,
        setCenter,
      },
      overviewImage: {
        getCurrentOverviewImage,
        getOverviewImages,
        hideOverviewImageModal,
        selectOverviewImage,
        showOverviewImageModal,
      },
      overlays: {
        data: {
          getDataOverlays,
          getVisibleDataOverlays,
        },
        showDataOverlay,
        hideDataOverlay,
        removeDataOverlay,
        addDataOverlay,
      },
      project: {
        data: {
          getProjectId,
          getName,
          getVersion,
          getDisease,
          getOrganism,
          getApiUrls,
        },
      },
    };

    const unsubscribe = store.subscribe(() => {
      const configurationStore = store.getState().configuration.main.data;

      if (configurationStore) {
        const configuration = configurationMapper(configurationStore);

        window.minerva = {
          ...window.minerva,
          configuration,
        };
      }
    });

    return unsubscribe;
  },

  registerPlugin({ pluginName, pluginVersion, pluginUrl }) {
    const hash = PluginsManager.hashedPlugins[pluginUrl];

    if (!hash) {
      throw new Error(ERROR_PLUGIN_URL_MISMATCH);
    }

    store.dispatch(
      registerPlugin({
        hash,
        isPublic: false,
        pluginName,
        pluginUrl,
        pluginVersion,
      }),
    );

    const element = PluginsManager.createAndGetPluginContent({ hash });

    return {
      element,
      events: {
        addListener: PluginsEventBus.addListener.bind(this, hash, pluginName),
        removeListener: PluginsEventBus.removeListener.bind(this, hash),
        removeAllListeners: PluginsEventBus.removeAllListeners.bind(this, hash),
      },
      legend: {
        setLegend: setLegend.bind(this, hash),
        removeLegend: removeLegend.bind(this, hash),
      },
    };
  },
  createAndGetPluginContent({ hash }) {
    const element = document.createElement('div');
    element.setAttribute(PLUGINS_CONTENT_ELEMENT_ATTR_NAME, hash);

    const wrapper = document.querySelector(`#${PLUGINS_CONTENT_ELEMENT_ID}`);
    wrapper?.append(element);

    return element;
  },
  removePluginContent({ hash }) {
    const elements = document.querySelectorAll(
      `div[${PLUGINS_CONTENT_ELEMENT_ATTR_NAME}="${hash}"]`,
    );

    elements.forEach(element => element.remove());
  },
};
